@@ -68,6 +68,8 @@ pub(crate) enum Tailrec {
 pub(crate) enum Term<T = TermId> {
     /// Identity (`.`)
     Id,
+    /// Recursion (`..`)
+    Recurse,
     ToString,
 
     Int(isize),
@@ -213,8 +215,7 @@ pub struct Compiler<S, F> {
     errs: Vec<Error<S>>,
 }
 
-// TODO: remove S: Default
-impl<S: Default, F> Default for Compiler<S, F> {
+impl<S, F> Default for Compiler<S, F> {
     fn default() -> Self {
         Self {
             lut: Lut::default(),
@@ -627,7 +628,7 @@ impl<'s, F> Compiler<&'s str, F> {
         use parse::Term::*;
         match t {
             Id => Term::Id,
-            Recurse => self.term(Call("!recurse", Vec::new()), &Tr::new()),
+            Recurse => Term::Recurse,
             Arr(t) => Term::Arr(self.iterm(t.map_or_else(|| Call("!empty", Vec::new()), |t| *t))),
             Neg(t) => Term::Neg(self.iterm(*t)),
             Pipe(l, None, r) => Term::Pipe(self.iterm(*l), None, self.iterm_tr(*r, tr)),
@@ -671,15 +672,9 @@ impl<'s, F> Compiler<&'s str, F> {
                 t
             }
             Num(n) => n.parse().map_or_else(|_| Term::Num(n.into()), Term::Int),
-            // map `try f catch g` to `label $x | try f catch (g, break $x)`
-            // and `try f` or `f?` to `label $x | try f catch (   break $x)`
             TryCatch(try_, catch) => {
-                let catch = match catch {
-                    None => Break(""),
-                    Some(c) => BinOp(c, parse::BinaryOp::Comma, Break("").into()),
-                };
-                let tc = self.with_label("", |c| Term::TryCatch(c.iterm(*try_), c.iterm(catch)));
-                Term::Label(self.lut.insert_term(tc))
+                let catch = catch.map_or_else(|| Call("!empty", Vec::new()), |t| *t);
+                Term::TryCatch(self.iterm(*try_), self.iterm(catch))
             }
             Fold(name, xs, pat, args) => {
                 use self::Fold::{Foreach, Reduce};
