@@ -25,6 +25,11 @@ type Results<'a, T, V> = box_iter::Results<'a, T, Exn<'a, V>>;
 pub(crate) struct Vars<'a, V>(RcList<Bind<V, usize, (&'a Id, Self)>>);
 
 impl<'a, V> Vars<'a, V> {
+    /// Initialise new variables from values.
+    pub fn new(vars: impl IntoIterator<Item = V>) -> Self {
+        Self(RcList::new().extend(vars.into_iter().map(Bind::Var)))
+    }
+
     fn get(&self, i: usize) -> Option<&Bind<V, usize, (&'a Id, Self)>> {
         self.0.get(i)
     }
@@ -249,6 +254,25 @@ fn lazy_is_lazy() {
     assert_eq!(iter.next(), Some(0));
 }
 
+/// Runs `def recurse: ., (.[]? | recurse); v | recurse`.
+fn recurse_run<'a, V: ValT + 'a>(v: V) -> ValXs<'a, V> {
+    let id = core::iter::once(Ok(v.clone()));
+    Box::new(id.chain(v.values().flat_map(Result::ok).flat_map(recurse_run)))
+}
+
+/// Runs `def recurse: (.[]? | recurse), .; v | recurse |= f`.
+///
+/// This uses a `recurse` different from that of `recurse_run`; in particular,
+/// `recurse_run` yields values from root to leafs, whereas
+/// this function performs updates from leafs to root.
+/// Performing updates from root to leafs can easily lead to
+/// nontermination or other weird effects, such as
+/// trying to update values that have been deleted.
+fn recurse_update<'a, V: ValT + 'a>(v: V, f: &dyn Update<'a, V>) -> ValXs<'a, V> {
+    use crate::path::Opt::Optional;
+    box_iter::then(v.map_values(Optional, |v| recurse_update(v, f)), f)
+}
+
 /// Combination of context and input value.
 pub type Cv<'c, V> = (Ctx<'c, V>, V);
 
@@ -309,6 +333,7 @@ impl<F: FilterT<F>> FilterT<F> for Id {
         use core::iter::once;
         match &lut.terms[self.0] {
             Ast::Id => box_once(Ok(cv.1)),
+            Ast::Recurse => recurse_run(cv.1),
             Ast::ToString => box_once(match cv.1.as_str() {
                 Some(_) => Ok(cv.1),
                 None => Ok(Self::V::from(cv.1.to_string())),
@@ -321,11 +346,18 @@ impl<F: FilterT<F>> FilterT<F> for Id {
             Ast::ObjSingle(k, v) => Box::new(
                 Self::cartesian(k, v, lut, cv).map(|(k, v)| Ok(Self::V::from_map([(k?, v?)])?)),
             ),
-            // TODO: write test for `try (break $x)`
             Ast::TryCatch(f, c) => {
-                Box::new(f.run(lut, (cv.0.clone(), cv.1)).flat_map(move |y| match y {
-                    Err(Exn(exn::Inner::Err(e))) => c.run(lut, (cv.0.clone(), e.into_val())),
-                    y => box_once(y),
+                let mut ys = f.run(lut, (cv.0.clone(), cv.1));
+                let mut end: Option<ValXs<_>> = None;
+                Box::new(core::iter::from_fn(move || match &mut end {
+                    Some(end) => end.next(),
+                    None => match ys.next()? {
+                        Err(Exn(exn::Inner::Err(e))) => {
+                            end = Some(c.run(lut, (cv.0.clone(), e.into_val())));
+                            end.as_mut().and_then(|end| end.next())
+                        }
+                        y => Some(y),
+                    },
                 }))
             }
             Ast::Neg(f) => Box::new(f.run(lut, cv).map(|v| Ok((-v?)?))),
@@ -453,7 +485,7 @@ impl<F: FilterT<F>> FilterT<F> for Id {
                 }
             }
             Ast::Native(id, args) => {
-                let cvs = bind_vars(args, lut, Ctx::new([], cv.0.inputs), cv);
+                let cvs = bind_vars(args, lut, cv.0.with_vars(Vars::new([])), cv);
                 flat_map_then(cvs, |cv| lut.funs[*id].run(lut, cv))
             }
             Ast::Label(id) => {
@@ -485,6 +517,7 @@ impl<F: FilterT<F>> FilterT<F> for Id {
             Ast::TryCatch(..) | Ast::Label(..) => err,
 
             Ast::Id => f(cv.1),
+            Ast::Recurse => recurse_update(cv.1, &f),
             Ast::Path(l, path) => {
                 let path = path.map_ref(|i| {
                     let cv = cv.clone();
@@ -537,7 +570,7 @@ impl<F: FilterT<F>> FilterT<F> for Id {
             }
             Ast::Native(id, args) => {
                 let init = cv.1.clone();
-                let cvs = bind_vars(args, lut, Ctx::new([], cv.0.inputs), cv);
+                let cvs = bind_vars(args, lut, cv.0.with_vars(Vars::new([])), cv);
                 reduce(cvs, init, move |cv, v| {
                     lut.funs[*id].update(lut, (cv.0, v), f.clone())
                 })
