@@ -4,6 +4,7 @@ use super::lex::{StrPart, Tok, Token};
 use super::path::{self, Path};
 use super::{ops, prec_climb};
 use alloc::{boxed::Box, vec::Vec};
+use core::fmt::{self, Debug};
 
 /// Parse error, storing what we expected and what we got instead.
 pub type Error<S, T = S> = (Expect<S>, T);
@@ -179,21 +180,6 @@ impl<S> Term<S> {
         Self::Str(None, [StrPart::Str(s)].into())
     }
 
-    /// `..`, also known as `recurse/0`, is defined as `., (.[]? | ..)`.
-    pub(crate) fn recurse(recurse: S) -> Self {
-        // `[]?`
-        let path = (path::Part::Range(None, None), path::Opt::Optional);
-        // `.[]?` (returns array/object elements or nothing instead)
-        let path = Term::Path(Term::Id.into(), Path(Vec::from([path])));
-
-        // `..`
-        let f = Term::Call(recurse, Vec::new());
-        // .[]? | ..
-        let pipe = Term::Pipe(path.into(), None, f.into());
-        // ., (.[]? | ..)
-        Term::BinOp(Term::Id.into(), BinaryOp::Comma, pipe.into())
-    }
-
     /// `{}[]` returns zero values.
     pub(crate) fn empty() -> Self {
         // `[]`
@@ -828,7 +814,7 @@ pub(crate) struct Module<S, B> {
 /// def map(f): [.[] | f];
 /// def recurse(f; cond): recurse(f | select(cond));
 /// ~~~
-#[derive(Debug)]
+// TODO v3.0: Make this just a tuple?
 pub struct Def<S, F = Term<S>> {
     /// name, e.g. `"double"` or `"map"`
     pub name: S,
@@ -838,6 +824,13 @@ pub struct Def<S, F = Term<S>> {
     pub body: F,
 }
 
+// required for Haskell interop
+impl<S: Debug, F: Debug> Debug for Def<S, F> {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        write!(f, "({:?}, {:?}, {:?})", self.name, self.args, self.body)
+    }
+}
+
 impl<S, F> Def<S, F> {
     pub(crate) fn new(name: S, args: Vec<S>, body: F) -> Self {
         Self { name, args, body }
