@@ -395,6 +395,12 @@ fn base() -> Box<[Filter<RunPtr<Val>>]> {
                 x.indices(&v).map(|idxs| idxs.map(to_int).collect())
             })
         }),
+        ("bsearch", v(1), |_, cv| {
+            let to_idx = |r: Result<_, _>| r.map_or_else(|i| -1 - i as isize, |i| i as isize);
+            unary(cv, move |a, x| {
+                a.as_arr().map(|a| Val::Int(to_idx(a.binary_search(&x))))
+            })
+        }),
     ])
 }
 
@@ -502,6 +508,13 @@ impl Val {
         }
     }
 
+    fn as_arr(&self) -> Result<&Rc<Vec<Self>>, Error> {
+        match self {
+            Self::Arr(a) => Ok(a),
+            _ => Err(Error::typ(self.clone(), Type::Arr.as_str())),
+        }
+    }
+
     /// Try to parse a string to a [`Self::Float`], else return [`Self::Null`].
     fn from_dec_str(n: &str) -> Self {
         n.parse().map_or(Self::Null, Self::Float)
